#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <string>
#include <vector>
#include <algorithm>
#include <queue>
#include <Windows.h>
using namespace std;

#define FOR(i, n) for (int i = 0; i < (int)(n); i++)
#define REP(i, v) for (int i = 0; i < (int)((v).size()); i++)
#define RL(i, st) for (int i = 0; i < (int)((s).length()); i++)
const int MINMAXSPENT =  5; // max gold spent per travel
const int MAXMAXSPENT = 20; // max gold spent per travel
const int MAXLEVELS = MINMAXSPENT + 1 + MINMAXSPENT + 1 + 9;
const int MAXDIJKSTRA = MAXLEVELS * 100 * 100 + 2;
const int INF = 999666111;
const int TIMELIMIT = 4500;
DWORD startClk;

#ifndef _MSC_VER
	inline int abs(int x) { return x < 0 ? -x : x; }
#endif

struct Pt {
	int x, y;
	Pt() {}
	Pt(int x, int y): x(x), y(y) {}
	void operator += (const Pt& rhs) { x += rhs.x; y += rhs.y; }
	void operator -= (const Pt& rhs) { x -= rhs.x; y -= rhs.y; }
	int length() const { return (abs(x) + abs(y)); }
	void inc(int dir, int amount = 1);
	void dec(int dir, int amount = 1);
};

const Pt DIR[4] = { Pt(0, -1), Pt(1, 0), Pt(0, 1), Pt(-1, 0) };


Pt operator + (const Pt& a, const Pt& b) { return Pt(a.x + b.x, a.y + b.y); }
Pt operator - (const Pt& a, const Pt& b) { return Pt(a.x - b.x, a.y - b.y); }
Pt operator * (const Pt& a, int m) { return Pt(a.x * m, a.y * m); }
void Pt::inc(int dir, int amount)
{
	(*this) += DIR[dir - 1] * amount;
}
void Pt::dec(int dir, int amount)
{
	(*this) -= DIR[dir - 1] * amount;
}


enum {
	Planet,
	Gold,
	Hole,
	Star,
	Empty,
};

struct FieldDesc {
	bool dark;
	Pt self;
	Pt teleDest;
	int type;
	int goldLeft;
	inline bool passable() { return type == Gold || type == Empty; }
	void init(Pt s)
	{
		dark = false;
		self = s;
		teleDest = Pt(-1,-1);
		type = Empty;
		goldLeft = 0;
	}
};


int w, h;
int startGold;
int K;
FieldDesc a[105][105];
int s[105][105];
Pt milky1, milky2;

static inline bool in(const Pt& p)
{
	return p.x >= 0 && p.x < w && p.y >= 0 && p.y < h;
}

static inline bool inTarget(const Pt& p)
{
	return p.x >= milky1.x && p.x < milky2.x && p.y >= milky1.y && p.y < milky2.y;
}

void readInput(void)
{
	FILE* f = fopen("cosmos.in", "rt");
	fscanf(f, "%d%d%d", &h, &w, &startGold);
	char t[200];
	FOR(y, h) {
		fscanf(f, "%s", t);
		FOR(x, w) {
			FieldDesc& f = a[y][x];
			s[y][x] = 0;
			f.init(Pt(x, y));
			switch (t[x]) {
				case '_': f.dark = true; break;

				case '0': f.type = Gold; f.goldLeft = 0; break;
				case '1': f.type = Gold; f.goldLeft = 1; break;
				case '2': f.type = Gold; f.goldLeft = 2; break;
				case '3': f.type = Gold; f.goldLeft = 3; break;
				case '4': f.type = Gold; f.goldLeft = 4; break;
				case '5': f.type = Gold; f.goldLeft = 5; break;
				case '6': f.type = Gold; f.goldLeft = 6; break;
				case '7': f.type = Gold; f.goldLeft = 7; break;
				case '8': f.type = Gold; f.goldLeft = 8; break;
				case '9': f.type = Gold; f.goldLeft = 9; break;
				case ')': f.type = Gold; f.goldLeft = 0; f.dark = true; break;

				case '!': f.type = Gold; f.goldLeft = 1; f.dark = true; break;
				case '@': f.type = Gold; f.goldLeft = 2; f.dark = true; break;
				case '#': f.type = Gold; f.goldLeft = 3; f.dark = true; break;
				case '$': f.type = Gold; f.goldLeft = 4; f.dark = true; break;
				case '%': f.type = Gold; f.goldLeft = 5; f.dark = true; break;
				case '^': f.type = Gold; f.goldLeft = 6; f.dark = true; break;
				case '&': f.type = Gold; f.goldLeft = 7; f.dark = true; break;
				case '*': f.type = Gold; f.goldLeft = 8; f.dark = true; break;
				case '(': f.type = Gold; f.goldLeft = 9; f.dark = true; break;

				case 's': f.type = Star; break;
				case 'S': f.type = Star; f.dark = true; break;

				case 't': K++; break;
				case 'T': K++; f.dark = true; break;

				case 'b': f.type = Hole; break;
				case 'B': f.type = Hole; f.dark = true; break;

				case 'p': f.type = Planet; break;
				case 'P': f.type = Planet; f.dark = true; break;

				case 'h': s[y][x] = 1; break;
				case 'H': s[y][x] = 1; f.dark = true; break;
			}
		}
	}
	fscanf(f, "%d%d%d%d", &milky1.y, &milky1.x, &milky2.y, &milky2.x);
	milky2 += Pt(1, 1);
	FOR(i, K/2) {
		Pt t1, t2;
		fscanf(f, "%d%d%d%d", &t1.y, &t1.x, &t2.y, &t2.x);
		a[t1.y][t1.x].teleDest = t2;
		a[t2.y][t2.x].teleDest = t1;
	}
	fclose(f);
}

struct Out {
	Pt p;
	int dir;
	int moveType;
};

vector<vector<Out> > sol;

void writeOutput(void)
{
	FILE* f = fopen("cosmos.out", "wt");
	int O = 0;
	REP(i, sol) O += (int) sol[i].size();
	fprintf(f, "%d\n", O);
	REP(i, sol) REP(j, sol[i])
		fprintf(f, "%d %d %d %d %d %d\n", sol[i][j].p.y, sol[i][j].p.x, sol[i][j].p.y, sol[i][j].p.x, sol[i][j].dir, sol[i][j].moveType);
	fclose(f);
}

int n, m, gold;

int cellsRequired(int maxspent)
{
	int levels = maxspent + 1 + maxspent + 1 + 9;
	return levels * w * h + 2;
}

struct Edge {
	int target, cost, dir, moveType;
	Edge(){}
	Edge(int target, int cost, int dir, int moveType = 1): target(target), cost(cost), dir(dir), moveType(moveType) {}
};

vector<Edge> g[MAXDIJKSTRA];

inline int enc(Pt p, int level)
{
	return level * w * h + p.y * w + p.x;
}

inline void decode(int idx, Pt& p, int& level)
{
	level = idx / (w * h);
	idx %= (w * h);
	p.y = idx / w;
	p.x = idx % w;
}

void addG(int src, Pt dest, int destLevel, int cost)
{
	g[src].push_back(Edge(enc(dest, destLevel), cost, 0));
}

void addG(Pt src, int srcLevel, int dest, int cost)
{
	g[enc(src, srcLevel)].push_back(Edge(dest, cost, 0));
}

void addG(Pt src, int srcLevel, Pt dest, int destLevel, int cost, int dir, int moveType)
{
	g[enc(src, srcLevel)].push_back(Edge(enc(dest, destLevel), cost, dir, moveType));
}

bool starNearby[102][102];
bool holeNearby[102][102];
int ds[MAXDIJKSTRA];
int from[MAXDIJKSTRA][3];

double coeffA, coeffB = 1, coeffC = -5;

double eval(int x, int y, int distance, int level)
{
	int balance;
	if (level <= m)
		balance = level - m;
	else
		balance = level - (2 * m + 1);

	double score = -1 * coeffA + distance * coeffB + balance * coeffC;
	// add closeness to the milkyway center score
	double dx = min(x - milky1.x, milky2.x - 1 - x);
	double dy = min(y - milky1.y, milky2.y - 1 - y);
	double scoreCloseness = -dx/2 + -dy/2;
	score += scoreCloseness;
	// if the final landing location is a teleport, get the fuck out:
	if (a[y][x].teleDest.x != -1) score += 10;
	return score;
}

void postprocessBecauseOfBuggyChecker(vector<Out>& path)
{
	for (int i = (int) path.size() - 1; i >= 0; i--) if (path[i].moveType < 0) {
		Out out1 = path[i], out2 = path[i];
		out2.p = out2.p - DIR[out2.dir - 1] * path[i].moveType;
		out2.moveType = 0;
		out1.moveType = -out1.moveType;
		path.erase(path.begin() + i);
		path.insert(path.begin() + i, out2);
		path.insert(path.begin() + i, out1);
	}
}

int decodeBalance(int level)
{
	bool hasMined = level > m;
	return hasMined ? (level - (2*m + 1)) : (level - m);
}

double augment(vector<Out>& stored)
{
	FOR(i, n) g[i].clear();
	int notMined = m;
	int mined = 2*m + 1;
	/*
	  level m - 7  : no gold is mined, but 7 is spent
	  level m - 6  : no gold is mined, but 6 is spent...
	  ...
	  level m      : no gold is mined and none is spent
	  level m + 1  : gold was mined, but balance is -7
	  level m + 2  : gold was mined, but balance is -6
	  ...
	  level 2*m + 1: gold was mined and the balance is 0
	  level 2*m + 2: gold was mined and the balance is 1
	  ...
	  level 2*m +10: gold was mined and the balance is 9
	*/
	int maxLevels = 2*m+11;
	n = cellsRequired(m);
	int SSRC = n - 2;
	int SSINK = n - 1;
	//
	vector<Pt> shipPos;
	FOR(y, h) FOR(x, w) if (s[y][x] && !inTarget(Pt(x, y))) {
		shipPos.push_back(Pt(x, y));
	}
	REP(i, shipPos) {
		addG(SSRC, shipPos[i], notMined, 0);
	}
	for (int y = milky1.y; y < milky2.y; y++)
		for (int x = milky1.x; x < milky2.x; x++)
			if (!s[y][x]) {
				FOR(l, maxLevels) {
					addG(Pt(x, y), l, SSINK, 0);
				}
			}
	int minBalance = max(-m, -gold);
	FOR(l, maxLevels) FOR(y, h) FOR(x, w) {
		bool hasMined = l > m;
		int balance = decodeBalance(l);
		if (balance < minBalance) continue;
		if (!a[y][x].passable() || holeNearby[y][x]) continue;
		int maxDist = 1 + (int) starNearby[y][x];
		// check for regular moves, at speed 1 or 2
		Pt src(x, y);
		for (int dist = 1; dist <= maxDist; dist++) {
			FOR(dr, 4) {
				Pt dest = src + DIR[dr]*dist;
				Pt mid = src + DIR[dr]*(dist/2);
				if (!in(dest)) continue;
				// jumping out of the dark?
				if (a[y][x].dark && !a[dest.y][dest.x].dark) continue;
				// if jumping from dark, the in-between cannot be non-dark, too:
				if (dist == 2 && a[y][x].dark && !a[mid.y][mid.x].dark) continue; 
				if (a[dest.y][dest.x].passable() && !holeNearby[dest.y][dest.x] && !s[dest.y][dest.x]) {
					addG(src, l, dest, l, 1, dr, dist);
					// check if it's a gold planet there:
					if (!hasMined && a[dest.y][dest.x].goldLeft > 0) {
						if (dest.x == 69 && dest.y == 43) {
							printf("Boo!\n");
						}
						int newBalance = balance + a[dest.y][dest.x].goldLeft;
						assert(mined + newBalance < maxLevels);
						addG(src, l, dest, mined + newBalance, 1, dr, dist);
					}
					// check if it's a teleport there:
					if (a[dest.y][dest.x].teleDest.x != -1) {
						Pt dest2 = a[dest.y][dest.x].teleDest;
						int neededGold = a[dest.y][dest.x].dark ? 0 : 1;
						if (!holeNearby[dest2.y][dest2.x] && balance - neededGold >= minBalance) {
							int newBalance = balance - neededGold;
							int newLevel = hasMined ? mined + newBalance : notMined + newBalance;
							assert(newLevel >= 0 && newLevel < maxLevels);
							addG(src, l, dest2, newLevel, 1, dr, -dist);
						}
					}
				}
			}
		}
	}
	queue<int> q;
	FOR(i, n) {
		ds[i] = INF;
		from[i][0] = -1;
	}
	q.push(SSRC);
	ds[SSRC] = 0;
	from[SSRC][0] = -2;
	from[SSRC][1] = 1;
	while (!q.empty()) {
		int idx = q.front();
		Pt srcP; int srcL;
		decode(idx, srcP, srcL);
		q.pop();
		REP(i, g[idx]) {
			int dest = g[idx][i].target;
			Pt destP; int destL;
			decode(dest, destP, destL);
			int mdist = ds[idx] + g[idx][i].cost;
			if (mdist < ds[dest]) {
				ds[dest] = mdist;
				from[dest][0] = idx;
				from[dest][1] = g[idx][i].dir;
				from[dest][2] = g[idx][i].moveType;
				q.push(dest);
			}
		}
	}
	if (ds[SSINK] >= INF) {
		return INF;
	}
	// choose best landing location:
	double bestScore = 1e99;
	int bl, by, bx;
	FOR(l, maxLevels)
		for (int y = milky1.y; y < milky2.y; y++)
			for (int x = milky1.x; x < milky2.x; x++) if (!s[y][x] && ds[enc(Pt(x, y), l)] < INF) {
				double score = eval(x, y, ds[enc(Pt(x, y), l)], l);
				if (score < bestScore) {
					bestScore = score;
					bl = l;
					by = y;
					bx = x;
				}
			}
	if (bestScore >= 1e99) return INF;
	vector<Out> path;
	Pt final(bx, by);
	Pt p = final;
	int cl = bl;
	while (true) {
		int idx = from[enc(p, cl)][0];
		if (idx >= SSRC || idx < 0) break;
		Out out;	
		out.dir = from[enc(p, cl)][1] + 1;
		out.moveType = from[enc(p, cl)][2];
		bool oldMining = cl > m;
		Pt oldP = p;
		decode(idx, p, cl);
		bool newMining = cl > m;
		if (!newMining && oldMining) {
#ifdef _DEBUG
			printf("Collected %d gold from gold mine at (%d, %d)\n", a[oldP.y][oldP.x].goldLeft, oldP.x, oldP.y);
#endif
			assert(a[oldP.y][oldP.x].goldLeft > 0);
			a[oldP.y][oldP.x].goldLeft = 0;
		}
		out.p = p;
		path.push_back(out);
	}
#ifdef _DEBUG
	static int shipsSaved = 0;
	printf("#%02d: ", shipsSaved++);
	printf("Moving ship from (%2d, %2d) to (%2d, %2d), ", p.x, p.y, final.x, final.y);
	if (bl > m) printf("MINING, final balance is %d\n", bl - (2*m + 1));
	else printf("NOT MINING, final balance is %d\n", bl - m);
#endif
	s[final.y][final.x] = 1;
	s[p.y][p.x] = 0;
	stored = path;
	reverse(stored.begin(), stored.end());
	//postprocessBecauseOfBuggyChecker(stored);
	int endBalance = decodeBalance(bl);
	gold += endBalance;
	return bestScore;
}


void precalc(void)
{
	coeffA = sqrt((double) w * h);
	memset(starNearby, 0, sizeof(starNearby));
	memset(holeNearby, 0, sizeof(holeNearby));
	FOR(y, h) FOR(x, w) {
		Pt src(x, y);
		for (int dy = -3; dy <= 3; dy++) {
			for (int dx = -3; dx <= 3; dx++) {
				Pt dest(x + dx, y + dy);
				if (in(dest) && a[dest.y][dest.x].type == Star && (src-dest).length() <= 3) {
					starNearby[y][x] = true;
				}
			}
		}
		for (int dy = -1; dy <= 1; dy++) {
			for (int dx = -1; dx <= 1; dx++) {
				Pt dest(x + dx, y + dy);
				if (in(dest) && a[dest.y][dest.x].type == Hole && (src-dest).length() <= 1) {
					holeNearby[y][x] = true;
				}
			}
		}
	}
}

void solve(void)
{
	precalc();
	gold = startGold;
	m = MINMAXSPENT;
	while (m < MAXMAXSPENT && cellsRequired(m) <= MAXDIJKSTRA) m++;
	m--;
	int shipsOut = 0;
	int N = 0;
	FOR(y, h) FOR(x, w) if (s[y][x]) {
		N++;
		if (!inTarget(Pt(x, y))) shipsOut++;
	}
	int shipsIn = N - shipsOut;
	while (shipsIn < N/2) {
		vector<Out> path;
		augment(path);
		sol.push_back(path);
		shipsIn++;
		shipsOut--;
	}
	while (shipsOut > 0) {
		if (GetTickCount() - startClk > TIMELIMIT) break;
		vector<Out> path;
		if (augment(path) < 0) {
			sol.push_back(path);
			shipsOut--;
		} else break;
	}
}

int main(void)
{
	startClk = GetTickCount();
	readInput();
	solve();
	writeOutput();
	return 0;
}
